@@ -11,18 +11,17 @@ import textwrap
 import types
 from contextlib import contextmanager
 from typing import Any, Dict, Union
+
 from pyccolo.ast_rewriter import AstRewriter
-from pyccolo.emit_event import _TRACER_STACK, allow_reentrant_event_handling, SkipAll
+from pyccolo.emit_event import _TRACER_STACK, SkipAll, allow_reentrant_event_handling
 from pyccolo.extra_builtins import make_guard_name
 from pyccolo.predicate import Predicate
-from pyccolo.syntax_augmentation import (
-    AugmentationSpec,
-    AugmentationType,
-)
+from pyccolo.syntax_augmentation import AugmentationSpec, AugmentationType
 from pyccolo.trace_events import TraceEvent
 from pyccolo.trace_stack import TraceStack
 from pyccolo.tracer import (
     BaseTracer,
+    NoopTracer,
     Null,
     Pass,
     Skip,
@@ -31,8 +30,6 @@ from pyccolo.tracer import (
     skip_when_tracing_disabled,
 )
 from pyccolo.utils import multi_context, resolve_tracer
-from pyccolo.version import __version__
-
 
 event = TraceEvent
 
@@ -40,8 +37,10 @@ event = TraceEvent
 call = TraceEvent.call
 return_ = TraceEvent.return_
 exception = TraceEvent.exception
+before_import = TraceEvent.before_import
 init_module = TraceEvent.init_module
 exit_module = TraceEvent.exit_module
+after_import = TraceEvent.after_import
 before_stmt = TraceEvent.before_stmt
 after_stmt = TraceEvent.after_stmt
 after_module_stmt = TraceEvent.after_module_stmt
@@ -70,7 +69,8 @@ before_lambda = TraceEvent.before_lambda
 after_lambda = TraceEvent.after_lambda
 before_call = TraceEvent.before_call
 after_call = TraceEvent.after_call
-argument = TraceEvent.argument
+before_argument = TraceEvent.before_argument
+after_argument = TraceEvent.after_argument
 before_return = TraceEvent.before_return
 after_return = TraceEvent.after_return
 before_dict_literal = TraceEvent.before_dict_literal
@@ -114,7 +114,7 @@ def tracer() -> BaseTracer:
     if len(_TRACER_STACK) > 0:
         return _TRACER_STACK[-1]
     else:
-        return BaseTracer()
+        return NoopTracer.instance()
 
 
 def instance() -> BaseTracer:
@@ -187,13 +187,18 @@ def tracing_context(tracers=None, *args, **kwargs):
 @contextmanager
 def tracing_enabled(tracers=None, **kwargs):
     tracers = _TRACER_STACK if tracers is None else tracers
+    if len(tracers) == 0:
+        raise ValueError("Expected at least one tracer to enable")
     with multi_context([tracer.tracing_enabled(**kwargs) for tracer in tracers]):
         yield
 
 
 @contextmanager
 def tracing_disabled(tracers=None, **kwargs):
-    tracers = _TRACER_STACK if tracers is None else tracers
+    if tracers is None:
+        tracers = _TRACER_STACK
+        if len(tracers) == 0:
+            tracers = [NoopTracer.instance()]
     with multi_context([tracer.tracing_disabled(**kwargs) for tracer in tracers]):
         yield
 
@@ -207,6 +212,7 @@ __all__ = [
     "AugmentationSpec",
     "AugmentationType",
     "BaseTracer",
+    "NoopTracer",
     "Null",
     "Pass",
     "Predicate",
@@ -236,3 +242,6 @@ __all__ = [
 
 # all the events now
 __all__.extend(evt.name for evt in TraceEvent)
+
+from . import _version  # noqa: E402
+__version__ = _version.get_versions()['version']
